@@ -1,18 +1,10 @@
 cpm.all <- 
   function(x, y, ...) 
 {   
-    if (!(is.data.frame(x))) {
-      stop("Input 'x' must be of class 'data.frame'. ")
-    }
-    if (!(is.numeric(y))) {
-      stop("Input 'y' must be of class 'numeric'.")
-    }
-    if (any(is.na(x))) {
-      stop("Input 'x' cannot contain missing values.")
-    }
-    if (any(is.na(y))) {
-      stop("Input 'y' cannot contain missing values.") 
-    }
+    if (!(is.data.frame(x))) {stop("Input 'x' must be of class 'data.frame'. ")}
+    if (!(is.numeric(y))) {stop("Input 'y' must be of class 'numeric'.")}
+    if (any(is.na(x))) {stop("Input 'x' cannot contain missing values.")}
+    if (any(is.na(y))) {stop("Input 'y' cannot contain missing values.")}
     if (nrow(x) != length(y)) {
       stop("Input 'x' must be same length as input 'y'.") 
     }
